#include <msp/gl/framebuffer.h>
#include <msp/gltk/layout.h>
#include "buttondemo.h"
#include "dropdowndemo.h"
#include "entrydemo.h"
#include "toggledemo.h"
#include "widgetdemo.h"

using namespace std;
using namespace Msp;

WidgetDemo::WidgetDemo(int, char **):
	window(800, 600),
	resources("basic.skin"),
	root(resources, window)
{
	window.set_title("GLtk widget demo");
	window.signal_close.connect(sigc::bind(sigc::mem_fun(this, &WidgetDemo::exit), 0));

	root.set_layout(new GLtk::Layout);

	root.add(selector);
	root.get_layout()->set_expand(selector, false, true);

	add_demo("Button", new ButtonDemo);
	add_demo("Dropdown", new DropdownDemo);
	add_demo("Entry", new EntryDemo);
	add_demo("Toggle", new ToggleDemo);
}

int WidgetDemo::main()
{
	window.show();
	return Application::main();
}

void WidgetDemo::tick()
{
	window.tick();
	GL::Framebuffer::system().clear(GL::COLOR_BUFFER_BIT);
	root.render();
	window.swap_buffers();
}

void WidgetDemo::add_demo(const string &title, GLtk::Panel *demo)
{
	demo->set_visible(false);
	root.add(*demo);
	root.get_layout()->add_constraint(*demo, GLtk::Layout::RIGHT_OF, selector);
	root.get_layout()->set_expand(*demo, true, true);

	selector.add_demo(title, demo);
}
