#include <msp/gl/meshbuilder.h>
#include "button.h"
#include "part.h"
#include "style.h"

using namespace std;

namespace Msp {
namespace GLtk {

Button::Button(const std::string &t):
	text(),
	icon(0),
	pressed(false)
{
	input_type = INPUT_NAVIGATION;
	set_text(t);
}

void Button::autosize_special(const Part &part, Geometry &ageom) const
{
	if(part.get_name()=="text")
		text.autosize(part, ageom);
	else if(part.get_name()=="icon" && icon)
	{
		const Sides &margin = part.get_margin();
		ageom.w = max(ageom.w, icon->get_width()+margin.left+margin.right);
		ageom.h = max(ageom.h, icon->get_height()+margin.top+margin.bottom);
	}
}

void Button::set_text(const std::string &t)
{
	text = t;
	signal_autosize_changed.emit();
	rebuild();
}

void Button::set_icon(const GL::Texture2D *i)
{
	icon = i;
	rebuild();
}

void Button::rebuild_special(const Part &part)
{
	if(part.get_name()=="text")
		text.build(part, state, geom, part_cache);
	if(part.get_name()=="icon")
	{
		if(icon)
		{
			Geometry rgeom;
			rgeom.w = icon->get_width();
			rgeom.h = icon->get_height();
			part.get_alignment().apply(rgeom, geom, part.get_margin());

			GL::MeshBuilder bld(part_cache.create_mesh(part, *icon));
			bld.color(1.0f, 1.0f, 1.0f);
			bld.begin(GL::TRIANGLE_STRIP);
			bld.texcoord(0, 1);
			bld.vertex(rgeom.x, rgeom.y+rgeom.h);
			bld.texcoord(0, 0);
			bld.vertex(rgeom.x, rgeom.y);
			bld.texcoord(1, 1);
			bld.vertex(rgeom.x+rgeom.w, rgeom.y+rgeom.h);
			bld.texcoord(1, 0);
			bld.vertex(rgeom.x+rgeom.w, rgeom.y);
			bld.end();
		}
	}
}

void Button::button_press(int, int, unsigned btn)
{
	if(btn==1)
	{
		pressed = true;
		set_state(ACTIVE);
	}
}

void Button::button_release(int x, int y, unsigned btn)
{
	if(pressed && btn==1)
	{
		clear_state(ACTIVE);
		pressed = false;

		if(geom.is_inside_relative(x, y))
			signal_clicked.emit();
	}
}

void Button::pointer_motion(int x, int y)
{
	if(pressed)
	{
		if(!geom.is_inside_relative(x, y))
			clear_state(ACTIVE);
		else
			set_state(ACTIVE);
	}
}

bool Button::navigate(Navigation nav)
{
	if(nav==NAV_ACTIVATE)
		signal_clicked.emit();
	else
		return false;

	return true;
}

void Button::on_style_change()
{
	text.set_style(style);
}


Button::Loader::Loader(Button &btn):
	DataFile::DerivedObjectLoader<Button, Widget::Loader>(btn)
{
	add("text", &Loader::text);
}

void Button::Loader::text(const std::string &t)
{
	obj.text = t;
}

} // namespace GLtk
} // namespace Msp
