#include "geometry.h"

namespace Msp {
namespace GLtk {

bool Geometry::is_inside(int x_, int y_) const
{
	return (x_>=x && x_<x+static_cast<int>(w) && y_>=y && y_<y+static_cast<int>(h));
}

bool Geometry::is_inside_relative(int x_, int y_) const
{
	return (x_>=0 && x_<static_cast<int>(w) && y_>=0 && y_<static_cast<int>(h));
}


Sides::Sides():
	top(0),
	right(0),
	bottom(0),
	left(0)
{ }

Sides::Sides(unsigned s):
	top(s),
	right(s),
	bottom(s),
	left(s)
{ }

Sides::Sides(unsigned v, unsigned h):
	top(v),
	right(h),
	bottom(v),
	left(h)
{ }

Sides::Sides(unsigned t, unsigned h, unsigned b):
	top(t),
	right(h),
	bottom(b),
	left(h)
{ }

Sides::Sides(unsigned t, unsigned r, unsigned b, unsigned l):
	top(t),
	right(r),
	bottom(b),
	left(l)
{ }


Sides::Loader::Loader(Sides &s):
	DataFile::ObjectLoader<Sides>(s)
{
	add("horizontal", &Loader::horizontal);
	add("vertical",   &Loader::vertical);
	add("top",    &Sides::top);
	add("right",  &Sides::right);
	add("bottom", &Sides::bottom);
	add("left",   &Sides::left);
}

void Sides::Loader::horizontal(unsigned h)
{
	obj.right = h;
	obj.left = h;
}

void Sides::Loader::vertical(unsigned v)
{
	obj.top = v;
	obj.bottom = v;
}


void Alignment::apply(Geometry &geom, const Geometry &parent) const
{
	if(parent.w>geom.w)
	{
		geom.w += static_cast<unsigned>((parent.w-geom.w)*w);
		geom.x += static_cast<int>((parent.w-geom.w)*x);
	}
	if(parent.h>geom.h)
	{
		geom.h += static_cast<unsigned>((parent.h-geom.h)*h);
		geom.y += static_cast<int>((parent.h-geom.h)*y);
	}
}

void Alignment::apply(Geometry &geom, const Geometry &parent, const Sides &margin) const
{
	Geometry content = parent;
	content.w -= margin.left+margin.right;
	content.h -= margin.bottom+margin.top;

	geom.x += margin.left;
	geom.y += margin.bottom;

	apply(geom, content);
}

} // namespace GLtk
} // namespace Msp
