#include "part.h"
#include "partcache.h"

using namespace std;

namespace Msp {
namespace GLtk {

CachedPart::CachedPart():
	part(0),
	texture(0),
	mesh(0)
{ }

CachedPart::~CachedPart()
{
	delete mesh;
}


PartCache::PartCache():
	rebuilding(false)
{ }

void PartCache::begin_rebuild()
{
	if(rebuilding)
		throw logic_error("nested rebuild");
	rebuilding = true;

	next = parts.begin();
	current = parts.end();
}

void PartCache::insert_special(const Part &part)
{
	if(part.get_name().empty())
		throw invalid_argument("PartCache::insert_special");
	if(!rebuilding)
		throw logic_error("!rebuilding");

	for(current=next; current!=parts.end(); ++current)
		if(current->part==&part)
		{
			parts.erase(next, current);
			break;
		}

	if(current==parts.end())
		current = parts.insert(next, CachedPart());
	else
		*current = CachedPart();
	current->part = &part;

	next = current;
	++next;
}

GL::Mesh &PartCache::create_mesh(const Part &part, const GL::Texture2D &tex)
{
	if(!rebuilding)
		throw logic_error("!rebuilding");

	if(current!=parts.end() && current->texture==&tex)
		return *current->mesh;

	for(current=next; current!=parts.end(); ++current)
		if(current->texture==&tex)
		{
			parts.erase(next, current);
			break;
		}

	if(current==parts.end())
	{
		current = parts.insert(next, CachedPart());
		current->texture = &tex;
		current->mesh = new GL::Mesh((GL::TEXCOORD2, GL::COLOR4_UBYTE, GL::VERTEX2));
	}
	else
		current->mesh->clear();
	current->part = &part;

	next = current;
	++next;

	return *current->mesh;
}

void PartCache::end_rebuild()
{
	rebuilding = false;
	parts.erase(next, parts.end());
}

} // namespace GLtk
} // namespace Msp
